# plugins/input_plugin.py

import logging
import threading
import pygame

class Plugin:
    def __init__(self, core, config):
        """
        Инициализация плагина ввода.

        :param core: Экземпляр ядра.
        :param config: Конфигурация плагина.
        """
        self.core = core
        self.config = config
        self.running = False
        self.input_thread = None
        self.setup()
        logging.info("Input-плагин инициализирован.")

    def setup(self):
        """Настройка плагина (регистрация хуков и API)."""
        self.core.register_hook('on_start', self.on_start)
        self.core.register_hook('on_exit', self.on_exit)
        logging.info("Input-плагин зарегистрировал хуки 'on_start' и 'on_exit'.")

        # Регистрация API метода для получения последнего события ввода
        self.last_input_event = None
        self.core.register_api('get_last_input_event', self.get_last_input_event)
        logging.info("API метод 'get_last_input_event' зарегистрирован.")

    def on_start(self):
        """Обработчик хука 'on_start' для запуска обработки ввода."""
        logging.info("Input-плагин получил событие 'on_start'. Запуск обработки ввода в отдельном потоке.")
        self.running = True
        self.input_thread = threading.Thread(target=self.run_input_loop, daemon=True)
        self.input_thread.start()

    def run_input_loop(self):
        """Цикл обработки событий ввода."""
        while self.running:
            events = pygame.event.get()
            for event in events:
                if event.type == pygame.KEYDOWN:
                    logging.info(f"Нажата клавиша: {pygame.key.name(event.key)}")
                    self.last_input_event = ('key_down', event.key)
                elif event.type == pygame.KEYUP:
                    logging.info(f"Отпущена клавиша: {pygame.key.name(event.key)}")
                    self.last_input_event = ('key_up', event.key)
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    logging.info(f"Нажата кнопка мыши: {event.button}")
                    self.last_input_event = ('mouse_button_down', event.button)
                elif event.type == pygame.MOUSEBUTTONUP:
                    logging.info(f"Отпущена кнопка мыши: {event.button}")
                    self.last_input_event = ('mouse_button_up', event.button)
                elif event.type == pygame.MOUSEMOTION:
                    logging.info(f"Движение мыши: {event.pos}")
                    self.last_input_event = ('mouse_motion', event.pos)
            pygame.time.wait(10)  # Пауза для снижения нагрузки на процессор

    def get_last_input_event(self):
        """Возвращает последнее событие ввода."""
        return self.last_input_event

    def on_exit(self):
        """Обработчик хука 'on_exit' для завершения работы плагина ввода."""
        logging.info("Input-плагин получил событие 'on_exit'. Остановка обработки ввода.")
        self.running = False
        if self.input_thread and self.input_thread.is_alive():
            self.input_thread.join()
            logging.info("Поток обработки ввода остановлен.")

    def unload(self):
        """Выгрузка плагина и отмена регистрации хуков и API."""
        self.core.unregister_hook('on_start', self.on_start)
        self.core.unregister_hook('on_exit', self.on_exit)
        self.core.unregister_api('get_last_input_event')
        logging.info("Input-плагин выгружен.")



from setuptools import setup, find_packages
import os

# Функция для чтения файлов
def read_file(filename):
    with open(filename, encoding='utf-8') as f:
        return f.read()

setup(
    name='SAGE_Core_X',
    version='0.1.1',  # Обновлённая версия
    description='Ядро системы плагинов для разработки расширяемых приложений.',
    long_description=read_file('README.md') + '\n\n' + read_file('CHANGELOG.md') + '\n\n' + read_file('DOC.md'),
    long_description_content_type='text/markdown',
    author='AGStudios',
    author_email='amckinatorgames@gmail.com',
    url='https://github.com/yourusername/sage-core',  # Замените на ваш URL репозитория
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)



# plugins/opengl_plugin.py

import logging
import pygame
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
import threading

class Plugin:
    def __init__(self, core, config):
        """
        Инициализация OpenGL-плагина.

        :param core: Экземпляр ядра.
        :param config: Конфигурация плагина.
        """
        self.core = core
        self.config = config
        self.running = False
        self.display_thread = None
        self.setup()
        logging.info("OpenGL-плагин инициализирован.")

    def setup(self):
        """Настройка плагина (регистрация хуков)."""
        self.core.register_hook('on_start', self.on_start)
        self.core.register_hook('on_exit', self.on_exit)
        logging.info("OpenGL-плагин зарегистрировал хуки 'on_start' и 'on_exit'.")

    def on_start(self):
        """Обработчик хука 'on_start' для запуска рендеринга OpenGL."""
        logging.info("OpenGL-плагин получил событие 'on_start'. Запуск OpenGL в отдельном потоке.")
        self.running = True
        self.display_thread = threading.Thread(target=self.run_display, daemon=True)
        self.display_thread.start()

    def run_display(self):
        """Настройка и запуск отображения OpenGL."""
        pygame.init()
        display = (800, 600)
        pygame.display.set_mode(display, DOUBLEBUF | OPENGL)
        pygame.display.set_caption("SAGE-Core-X OpenGL Plugin")

        glEnable(GL_DEPTH_TEST)
        gluPerspective(45, (display[0] / display[1]), 0.1, 50.0)
        glTranslatef(0.0, 0.0, -5)

        while self.running:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False

            glRotatef(1, 3, 1, 1)
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
            self.draw_cube()
            pygame.display.flip()
            pygame.time.wait(10)

        pygame.quit()
        logging.info("Цикл отображения OpenGL завершен.")

    def draw_cube(self):
        """Отрисовка куба."""
        glBegin(GL_QUADS)

        # Передняя грань (красная)
        glColor3f(1.0, 0.0, 0.0)
        glVertex3f( 1.0, 1.0,-1.0)
        glVertex3f(-1.0, 1.0,-1.0)
        glVertex3f(-1.0, 1.0, 1.0)
        glVertex3f( 1.0, 1.0, 1.0)

        # Задняя грань (зеленая)
        glColor3f(0.0,1.0,0.0)
        glVertex3f( 1.0,-1.0, 1.0)
        glVertex3f(-1.0,-1.0, 1.0)
        glVertex3f(-1.0,-1.0,-1.0)
        glVertex3f( 1.0,-1.0,-1.0)

        # Левая грань (синяя)
        glColor3f(0.0,0.0,1.0)
        glVertex3f(-1.0, 1.0, 1.0)
        glVertex3f(-1.0, 1.0,-1.0)
        glVertex3f(-1.0,-1.0,-1.0)
        glVertex3f(-1.0,-1.0, 1.0)

        # Правая грань (желтая)
        glColor3f(1.0,1.0,0.0)
        glVertex3f(1.0, 1.0,-1.0)
        glVertex3f(1.0, 1.0, 1.0)
        glVertex3f(1.0,-1.0, 1.0)
        glVertex3f(1.0,-1.0,-1.0)

        # Верхняя грань (циан)
        glColor3f(0.0,1.0,1.0)
        glVertex3f( 1.0, 1.0, 1.0)
        glVertex3f(-1.0, 1.0, 1.0)
        glVertex3f(-1.0,-1.0, 1.0)
        glVertex3f( 1.0,-1.0, 1.0)

        # Нижняя грань (пурпурная)
        glColor3f(1.0,0.0,1.0)
        glVertex3f( 1.0,-1.0,-1.0)
        glVertex3f(-1.0,-1.0,-1.0)
        glVertex3f(-1.0, 1.0,-1.0)
        glVertex3f( 1.0, 1.0,-1.0)

        glEnd()

    def on_exit(self):
        """Обработчик хука 'on_exit' для завершения работы OpenGL."""
        logging.info("OpenGL-плагин получил событие 'on_exit'. Остановка OpenGL.")
        self.running = False
        if self.display_thread and self.display_thread.is_alive():
            self.display_thread.join()
            logging.info("Поток отображения OpenGL остановлен.")

    def unload(self):
        """Выгрузка плагина и отмена регистрации хуков."""
        self.core.unregister_hook('on_start', self.on_start)
        self.core.unregister_hook('on_exit', self.on_exit)
        logging.info("OpenGL-плагин выгружен.")
